#define G_LOG_DOMAIN "boilerplate-plugins-activatable"

#include "config.h"

#include "boilerplate-plugins-activatable.h"
#include "boilerplate-application.h"

#include <glib.h>

/**
 * SECTION:boilerplate-plugins-activatable
 * @short_description: Interface for activatable extensions
 * @include: boilerplate-plugins-activatable.h
 * @see_also: #PeasExtensionSet
 *
 * #BoilerplatePluginsActivatable is an interface which should be implemented by
 * extensions that should be activated on an application.
 **/

G_DEFINE_INTERFACE (BoilerplatePluginsActivatable, boilerplate_plugins_activatable, G_TYPE_OBJECT)

/**
 * boilerplate_plugins_activatable_activate:
 * @activatable: A #BoilerplatePluginsActivatable.
 *
 * Activates the extension on the application.
 */
void
boilerplate_plugins_activatable_activate (BoilerplatePluginsActivatable *activatable)
{
    BoilerplatePluginsActivatableInterface *iface;

    g_return_if_fail (BOILERPLATE_IS_PLUGINS_ACTIVATABLE (activatable));

    iface = BOILERPLATE_PLUGINS_ACTIVATABLE_GET_IFACE (activatable);

    if (iface->activate != NULL)
    {
        iface->activate (activatable);
    }
}

/**
 * boilerplate_plugins_activatable_deactivate:
 * @activatable: A #BoilerplatePluginsActivatable.
 *
 * Deactivates the extension from the application.
 */
void
boilerplate_plugins_activatable_deactivate (BoilerplatePluginsActivatable *activatable)
{
    BoilerplatePluginsActivatableInterface *iface;

    g_return_if_fail (BOILERPLATE_IS_PLUGINS_ACTIVATABLE (activatable));

    iface = BOILERPLATE_PLUGINS_ACTIVATABLE_GET_IFACE (activatable);

    if (iface->deactivate != NULL)
    {
        iface->deactivate (activatable);
    }
}

static void
boilerplate_plugins_activatable_default_init (BoilerplatePluginsActivatableInterface *iface)
{
	/**
	 * BoilerplatePluginsActivatable:app:
	 *
	 * The application property contains the current application for this
	 * #BoilerplatePluginsActivatable instance.
	 */
	g_object_interface_install_property (iface,
	                                     g_param_spec_object ("application",
	                                                          "Application",
	                                                          "The current application",
	                                                          BOILERPLATE_TYPE_APPLICATION,
	                                                          G_PARAM_READWRITE |
	                                                          G_PARAM_CONSTRUCT_ONLY |
	                                                          G_PARAM_STATIC_STRINGS));
}
