#define G_LOG_DOMAIN "boilerplate-preference"

#include "config.h"

#include "boilerplate-preferences-page.h"

#include <gtk/gtk.h>

/**
 * SECTION:boilerplate-preferences-page
 * @title: BoilerplatePreferencesPage
 * @short_description: A #GtkBin instance created for #BoilerplatePreferences
 * @include: boilerplate-preferences-page.h
 * @see_also: #GtkBin, #BoilerplatePreferencesPage
 *
 * The #BoilerplatePreferencesPage is an instance #GtkBin and is the page
 * preferences will be dumped into in the Preferences window.
 **/

typedef struct _BoilerplatePreferencesPagePrivate
{
    GtkWidget *box;
} BoilerplatePreferencesPagePrivate;

struct _BoilerplatePreferencesPage
{
    GtkWindow parent_instance;
};

G_DEFINE_TYPE_WITH_PRIVATE (BoilerplatePreferencesPage, boilerplate_preferences_page, GTK_TYPE_BIN)

/*
 * boilerplate_preferences_page_add_page:
 * @object: An instance of #BoilerplatePreferencesPage
 *
 * Does any last minute clean-ups when the class ends.
 */
void
boilerplate_preferences_page_add_page (BoilerplatePreferencesPage *self,
                                  GtkWidget              *page,
                                  const gchar            *name,
                                  const gchar            *title)
{
    g_return_if_fail (BOILERPLATE_IS_PREFERENCES_PAGE (self));

    BoilerplatePreferencesPagePrivate *priv = boilerplate_preferences_page_get_instance_private (boilerplate_preferences_page (self));

    gtk_container_add_with_properties (GTK_CONTAINER (priv->pages), GTK_WIDGET (page),
                                       "name", name,
                                       "title", title,
                                       NULL);
}

/*
 * boilerplate_preferences_page_finalize:
 * @object: An instance of #BoilerplatePreferencesPage
 *
 * Does any last minute clean-ups when the class ends.
 */
static void
boilerplate_preferences_page_finalize (GObject *object)
{
    BoilerplatePreferencesPagePrivate *priv = boilerplate_preferences_page_get_instance_private (boilerplate_preferences_page (object));

    G_OBJECT_CLASS (boilerplate_preferences_page_parent_class)->finalize (object);
}

/*
 * boilerplate_preferences_page_init:
 * @self: An instance of #BoilerplatePreferencesPage
 *
 * Initializes various aspects of the window. Anything the window would need
 * before becoming presented should be done here.
 */
static void
boilerplate_preferences_page_init (BoilerplatePreferencesPage *self)
{
    BoilerplatePreferencesPagePrivate *priv = boilerplate_preferences_page_get_instance_private (self);

    gtk_widget_init_template (GTK_WIDGET (self));
}

/*
 * boilerplate_preferences_page_class_init:
 * @klass: An instance of #BoilerplatePreferencesPageClass
 *
 * Initializes the various virtural functions in #BoilerplatePreferencesPageClass.
 * In addtition it connects the various #GtkWidgets from the gresource file
 * using gtk_widget_class_set_template_from_resource().
 */
static void
boilerplate_preferences_page_class_init (BoilerplatePreferencesPageClass *klass)
{
    GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
    GObjectClass *object_class = G_OBJECT_CLASS (klass);

    object_class->finalize = boilerplate_preferences_page_finalize;

    gtk_widget_class_set_template_from_resource (widget_class,
                                                 PACKAGE_RESOURCES "/ui/preferences/page.ui");

    gtk_widget_class_bind_template_child_private (widget_class, BoilerplatePreferencesPage, box);
}

/**
 * boilerplate_preferences_page_new:
 * @window: A #GtkApplicationWindow instance
 *
 * Creates a new #BoilerplatePreferencesPage, a sub-object of #GtkWindow.
 *
 * ## Usage
 * |[
 * BoilerplatePreferencesPage *preferences;
 *
 * preferences = boilerplate_preferences_page_new (window);
 * ]|
 *
 * When creating a boilerplate_preferences_page_new() a #BoilerplateWindow will need
 * to be passed. This is because #BoilerplatePreferencesPage acts as a modal window
 * to the master window #BoilerplateWindow attached to the
 * #BoilerplateApplication.
 *
 * Returns: #BoilerplatePreferencesPage; a sub-object of #GtkWindow
 */
BoilerplatePreferencesPage *
boilerplate_preferences_page_new (void)
{
    return g_object_new (BOILERPLATE_TYPE_PREFERENCES_PAGE,
                         NULL);
}
